package de.berlin.airport.artemis;

import java.io.IOException;
import java.util.HashMap;

import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.ReflectionException;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;

import com.google.gson.Gson;

import de.berlin.airport.artemis.json.Connections;
import de.berlin.airport.artemis.json.Consumers;
import de.berlin.airport.artemis.json.Producers;
import de.berlin.airport.artemis.json.QueuePage;
import de.berlin.airport.artemis.json.Sessions;

public class JMXBroker {

    private JMXConnector jmxConnector;
    private MBeanServerConnection beanConnector;
    private ObjectName brokerBean;

    private String brokerName;

    public JMXBroker(String brokerName) {
        this.brokerName = brokerName;
    }

    public void connect(String host, int port, String username, String password) throws IOException, MalformedObjectNameException {
        HashMap<String, Object> jmxEnvironment = new HashMap<String, Object>();
        jmxEnvironment.put(JMXConnector.CREDENTIALS, new String[] { username, password });

        JMXServiceURL url = new JMXServiceURL(String.format("service:jmx:rmi:///jndi/rmi://%s:%d/jmxrmi", host, port));
        jmxConnector = JMXConnectorFactory.newJMXConnector(url, jmxEnvironment);

        jmxConnector.connect();
        beanConnector = jmxConnector.getMBeanServerConnection();
        brokerBean = new ObjectName(String.format("org.apache.activemq.artemis:broker=\"%s\"", brokerName));
    }

    public void disconnect() throws IOException {
        jmxConnector.close();
        beanConnector = null;
        brokerBean = null;
    }

    public JMXQueue getQueueJMX(String queueName) throws MalformedObjectNameException {
        return new JMXQueue(brokerBean.getCanonicalName(), queueName, beanConnector);
    }

    public Sessions getSessions() throws InstanceNotFoundException, MBeanException, ReflectionException, IOException {
        Object ret = beanConnector.invoke(brokerBean, "listAllSessionsAsJSON", null, null);
        Gson gson = new Gson();
        return gson.fromJson(ret.toString(), Sessions.class);
    }

    public Connections getConnections() throws InstanceNotFoundException, MBeanException, ReflectionException, IOException {
        Object ret = beanConnector.invoke(brokerBean, "listConnectionsAsJSON", null, null);
        Gson gson = new Gson();
        return gson.fromJson(ret.toString(), Connections.class);
    }

    public QueuePage getQueuesPaged(String options, int page, int size) throws InstanceNotFoundException, MBeanException, ReflectionException, IOException {
        Object[] parameters = new Object[3];
        parameters[0] = options;
        parameters[1] = page;
        parameters[2] = size;

        String[] signatures = new String[3];
        signatures[0] = String.class.getName();
        signatures[1] = int.class.getName();
        signatures[2] = int.class.getName();

        Object ret = beanConnector.invoke(brokerBean, "listQueues", parameters, signatures);
        Gson gson = new Gson();
        return gson.fromJson(ret.toString(), QueuePage.class);
    }

    public Consumers getConsumers(String connectionId) throws InstanceNotFoundException, MBeanException, ReflectionException, IOException {
        Object[] parameters = new Object[1];
        parameters[0] = connectionId;

        String[] signatures = new String[1];
        signatures[0] = String.class.getName();

        Object ret = beanConnector.invoke(brokerBean, "listConsumersAsJSON", parameters, signatures);
        Gson gson = new Gson();
        return gson.fromJson(ret.toString(), Consumers.class);
    }

    public Producers getProducers() throws InstanceNotFoundException, MBeanException, ReflectionException, IOException {
        Object ret = beanConnector.invoke(brokerBean, "listProducersInfoAsJSON", null, null);
        Gson gson = new Gson();
        return gson.fromJson(ret.toString(), Producers.class);
    }
}
